template <typename T>
struct HANDLE
{
private:
  int Handle;

  T Inputs;

public:
  HANDLE() : Handle(INVALID_HANDLE)
  {
  }

  bool operator ==( const T &Inputs2 ) const
  {
    return(this.Inputs == Inputs2);
  }

  void operator =( const T &Inputs2 )
  {
    this.Inputs = Inputs2;
  }

  int GetHandle()
  {
    return((this.Handle != INVALID_HANDLE) ? this.Handle : (this.Handle = this.Inputs.GetHandle()));
  }
};

template <typename T>
int GetHandle( HANDLE<T> &Handles[], const T &Inputs )
{
  const int Size = ArraySize(Handles);

  for (int i = 0; i < Size; i++)
    if (Handles[i] == Inputs)
      return(Handles[i].GetHandle());

  ArrayResize(Handles, Size + 1);
  Handles[Size] = Inputs;

  return(Handles[Size].GetHandle());
}

struct MACD
{
  string             symbol;
  ENUM_TIMEFRAMES    period;
  int                fast_ema_period;
  int                slow_ema_period;
  int                signal_period;
  ENUM_APPLIED_PRICE applied_price;

  MACD( void )
  {
  }

  MACD( const string             &isymbol,
        const ENUM_TIMEFRAMES    &iperiod,
        const int                &ifast_ema_period,
        const int                &islow_ema_period,
        const int                &isignal_period,
        const ENUM_APPLIED_PRICE &iapplied_price ) :
        symbol((isymbol == NULL) || (isymbol == "") ? _Symbol : isymbol),
        period(iperiod == PERIOD_CURRENT ? ::Period() : iperiod),
        fast_ema_period(ifast_ema_period),
        slow_ema_period(islow_ema_period),
        signal_period(isignal_period),
        applied_price(iapplied_price)
  {
  }

  int GetHandle( void ) const
  {
    return(::iMACD(this.symbol, this.period, this.fast_ema_period, this.slow_ema_period, this.signal_period, this.applied_price));
  }

  bool operator ==( const MACD &Inputs ) const
  {
    return((this.symbol == Inputs.symbol) &&
           (this.period == Inputs.period) &&
           (this.fast_ema_period == Inputs.fast_ema_period) &&
           (this.slow_ema_period == Inputs.slow_ema_period) &&
           (this.signal_period == Inputs.signal_period) &&
           (this.applied_price == Inputs.applied_price));
  }
};

struct MA
{
  string             symbol;
  ENUM_TIMEFRAMES    period;
  int                ma_period;
  int                ma_shift;
  ENUM_MA_METHOD     ma_method;
  ENUM_APPLIED_PRICE applied_price;

  MA( void )
  {
  }

  MA( const string             &isymbol,
      const ENUM_TIMEFRAMES    &iperiod,
      const int                &ima_period,
      const int                &ima_shift,
      const ENUM_MA_METHOD     &ima_method,
      const ENUM_APPLIED_PRICE &iapplied_price ) :
      symbol((isymbol == NULL) || (isymbol == "") ? _Symbol : isymbol),
      period(iperiod == PERIOD_CURRENT ? ::Period() : iperiod),
      ma_period(ima_period),
      ma_shift(ima_shift),
      ma_method(ima_method),
      applied_price(iapplied_price)
  {
  }

  int GetHandle( void ) const
  {
    return(::iMA(this.symbol, this.period, this.ma_period, this.ma_shift, this.ma_method, this.applied_price));
  }

  bool operator ==( const MA &Inputs ) const
  {
    return((this.symbol == Inputs.symbol) &&
           (this.period == Inputs.period) &&
           (this.ma_period == Inputs.ma_period) &&
           (this.ma_shift == Inputs.ma_shift) &&
           (this.ma_method == Inputs.ma_method) &&
           (this.applied_price == Inputs.applied_price));
  }
};

// MT5- iMACD.
int iMACD2( const string             symbol,
            const ENUM_TIMEFRAMES    period,
            const int                fast_ema_period,
            const int                slow_ema_period,
            const int                signal_period,
            const ENUM_APPLIED_PRICE applied_price )
{
  static HANDLE<MACD> Handles[];
  const MACD Inputs(symbol, period, fast_ema_period, slow_ema_period, signal_period, applied_price);

  return(GetHandle(Handles, Inputs));
}

// MT5- iMA.
int iMA2( const string             symbol,
          const ENUM_TIMEFRAMES    period,
          const int                ma_period,
          const int                ma_shift,
          const ENUM_MA_METHOD     ma_method,
          const ENUM_APPLIED_PRICE applied_price )
{
  static HANDLE<MA> Handles[];
  const MA Inputs(symbol, period, ma_period, ma_shift, ma_method, applied_price);

  return(GetHandle(Handles, Inputs));
}

//+------------------------------------------------------------------+
//|                                               IndicatorsMQL4.mqh |
//|                              Copyright  2018, Vladimir Karputov |
//|                                           http://wmua.ru/slesar/ |
//+------------------------------------------------------------------+
#property copyright "Copyright  2018, Vladimir Karputov"
#property link      "http://wmua.ru/slesar/"
#property version   "1.003"
double NaN=double("nan");
//+------------------------------------------------------------------+
//| iAC function in MQL4 notation                                    |
//+------------------------------------------------------------------+
double   iAC(
             string                       symbol,              // symbol name
             ENUM_TIMEFRAMES              timeframe,           // timeframe
             int                          shift                // shift
             )
  {
   double result=NaN;
//---
   int handle=iAC(symbol,timeframe);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iAD function in MQL4 notation                                    |
//+------------------------------------------------------------------+
double   iAD(
             string                       symbol,              // symbol name
             ENUM_TIMEFRAMES              timeframe,           // timeframe
             ENUM_APPLIED_VOLUME          applied_volume,      // volume type for calculation
             int                          shift                // shift
             )
  {
   double result=NaN;
//---
   int handle=iAD(symbol,timeframe,applied_volume);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iADX function in MQL4 notation                                   |
//|   The buffer numbers are the following:                          |
//|      MQL4 0 - MODE_MAIN, 1 - MODE_PLUSDI, 2 - MODE_MINUSDI       |
//|      MQL5 0 - MAIN_LINE, 1 - PLUSDI_LINE, 2 - MINUSDI_LINE       |
//+------------------------------------------------------------------+
double   iADX(
              string                      symbol,              // symbol name
              ENUM_TIMEFRAMES             timeframe,           // timeframe
              int                         adx_period,          // averaging period
              int                         buffer,              // buffer
              int                         shift                // shift
              )
  {
   double result=NaN;
//---
   int handle=iADX(symbol,timeframe,adx_period);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,buffer,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iADXWilder function in MQL4 notation                             |
//|   The buffer numbers are the following:                          |
//|      MQL4 0 -    -/-,    1 -      -/-,    2 -      -/-           |
//|      MQL5 0 - MAIN_LINE, 1 - PLUSDI_LINE, 2 - MINUSDI_LINE       |
//+------------------------------------------------------------------+
double   iADXWilder(
                    string                symbol,              // symbol name
                    ENUM_TIMEFRAMES       timeframe,           // timeframe
                    int                   adx_period,          // averaging period
                    int                   buffer,              // buffer
                    int                   shift                // shift
                    )
  {
   double result=NaN;
//---
   int handle=iADXWilder(symbol,timeframe,adx_period);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,buffer,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iAlligator function in MQL4 notation                             |
//|   The buffer numbers are the following:                          |
//|      MQL4 1 - MODE_GATORJAW, 2 - MODE_GATORTEETH, 3 - MODE_GATORLIPS   |
//|      MQL5 0 - GATORJAW_LINE, 1 - GATORTEETH_LINE, 2 - GATORLIPS_LINE   |
//+------------------------------------------------------------------+
double   iAlligator(
                    string                symbol,              // symbol name
                    ENUM_TIMEFRAMES       timeframe,           // timeframe
                    int                   jaw_period,          // period for the calculation of jaws
                    int                   jaw_shift,           // horizontal shift of jaws
                    int                   teeth_period,        // period for the calculation of teeth
                    int                   teeth_shift,         // horizontal shift of teeth
                    int                   lips_period,         // period for the calculation of lips
                    int                   lips_shift,          // horizontal shift of lips
                    ENUM_MA_METHOD        ma_method,           // type of smoothing
                    ENUM_APPLIED_PRICE    applied_price,       // type of price
                    int                   buffer,              // buffer
                    int                   shift                // shift
                    )
  {
//--- it is assumed that the input parameters are written in MQL4 transcription
   switch(buffer)
     {
      case  1:       // MQL4: MODE_GATORJAW
         buffer=0;   // MQL5: GATORJAW_LINE
         break;
      case  2:       // MQL4: MODE_GATORTEETH
         buffer=1;   // MQL5: GATORTEETH_LINE
         break;
      case  3:       // MQL4: MODE_GATORLIPS
         buffer=2;   // MQL5: GATORLIPS_LINE
         break;
     }
   double result=NaN;
//---
   int handle=iAlligator(symbol,timeframe,jaw_period,jaw_shift,teeth_period,teeth_shift,lips_period,lips_shift,
                         ma_method,applied_price);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,buffer,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iAMA function in MQL4 notation                                   |
//+------------------------------------------------------------------+
double   iAMA(
              string                      symbol,              // symbol name
              ENUM_TIMEFRAMES             timeframe,           // timeframe
              int                         ama_period,          // average period for AMA
              int                         fast_ma_period,      // fast MA period
              int                         slow_ma_period,      // slow MA period
              int                         ama_shift,           // horizontal shift of the indicator
              ENUM_APPLIED_PRICE          applied_price,       // type of the price
              int                         shift                // shift
              )
  {
   double result=NaN;
//---
   int handle=iAMA(symbol,timeframe,ama_period,fast_ma_period,slow_ma_period,ama_shift,applied_price);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iAMA function in MQL4 notation                                   |
//+------------------------------------------------------------------+
double   iAO(
             string                       symbol,              // symbol name
             ENUM_TIMEFRAMES              timeframe,           // timeframe
             int                          shift                // shift
             )
  {
   double result=NaN;
//---
   int handle=iAO(symbol,timeframe);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iATR function in MQL4 notation                                   |
//+------------------------------------------------------------------+
double   iATR(
              string                      symbol,              // symbol name
              ENUM_TIMEFRAMES             timeframe,           // timeframe
              int                         ma_period,           // averaging period
              int                         shift                // shift
              )
  {
   double result=NaN;
//---
   int handle=iATR(symbol,timeframe,ma_period);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iBearsPower function in MQL4 notation                            |
//+------------------------------------------------------------------+
double   iBearsPower(
                     string               symbol,              // symbol name
                     ENUM_TIMEFRAMES      timeframe,           // timeframe
                     int                  ma_period,           // averaging period
                     int                  shift                // shift
                     )
  {
   double result=NaN;
//---
   int handle=iBearsPower(symbol,timeframe,ma_period);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iBands function in MQL4 notation                                 |
//|   The buffer numbers are the following:                          |
//|      MQL4 0 - MODE_MAIN, 1 - MODE_UPPER, 2 - MODE_LOWER          |
//|      MQL5 0 - BASE_LINE, 1 - UPPER_BAND, 2 - LOWER_BAND          |
//+------------------------------------------------------------------+
double   iBands(
                string                    symbol,              // symbol name
                ENUM_TIMEFRAMES           timeframe,           // timeframe
                int                       bands_period,        // period for average line calculation
                int                       bands_shift,         // horizontal shift of the indicator
                double                    deviation,           // number of standard deviations
                ENUM_APPLIED_PRICE        applied_price,       // type of price
                int                       buffer,              // buffer
                int                       shift                // shift
                )
  {
   double result=NaN;
//---
   int handle=iBands(symbol,timeframe,bands_period,bands_shift,deviation,applied_price);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,buffer,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iBullsPower function in MQL4 notation                            |
//+------------------------------------------------------------------+
double   iBullsPower(
                     string               symbol,              // symbol name
                     ENUM_TIMEFRAMES      timeframe,           // timeframe
                     int                  ma_period,           // averaging period
                     int                  shift                // shift
                     )
  {
   double result=NaN;
//---
   int handle=iBullsPower(symbol,timeframe,ma_period);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iCCI function in MQL4 notation                                   |
//+------------------------------------------------------------------+
double   iCCI(
              string                      symbol,              // symbol name
              ENUM_TIMEFRAMES             timeframe,           // timeframe
              int                         ma_period,           // averaging period
              ENUM_APPLIED_PRICE          applied_price,       // type of price
              int                         shift                // shift
              )
  {
   double result=NaN;
//---
   int handle=iCCI(symbol,timeframe,ma_period,applied_price);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iChaikin function in MQL4 notation                               |
//+------------------------------------------------------------------+
double   iChaikin(
                  string                  symbol,              // symbol name
                  ENUM_TIMEFRAMES         timeframe,           // timeframe
                  int                     fast_ma_period,      // fast period
                  int                     slow_ma_period,      // slow period
                  ENUM_MA_METHOD          ma_method,           // smoothing type
                  ENUM_APPLIED_VOLUME     applied_volume,      // type of volume
                  int                     shift                // shift
                  )
  {
   double result=NaN;
//---
   int handle=iChaikin(symbol,timeframe,fast_ma_period,slow_ma_period,ma_method,applied_volume);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iDEMA function in MQL4 notation                                  |
//+------------------------------------------------------------------+
double   iDEMA(
               string                     symbol,              // symbol name
               ENUM_TIMEFRAMES            timeframe,           // timeframe
               int                        ma_period,           // averaging period
               int                        ma_shift,            // horizontal shift
               ENUM_APPLIED_PRICE         applied_price,       // type of price
               int                        shift                // shift
               )
  {
   double result=NaN;
//---
   int handle=iDEMA(symbol,timeframe,ma_period,ma_shift,applied_price);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iDeMarker function in MQL4 notation                              |
//+------------------------------------------------------------------+
double   iDeMarker(
                   string                 symbol,              // symbol name
                   ENUM_TIMEFRAMES        timeframe,           // timeframe
                   int                    ma_period,           // averaging period
                   int                    shift                // shif
                   )
  {
   double result=NaN;
//---
   int handle=iDeMarker(symbol,timeframe,ma_period);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iEnvelopes function in MQL4 notation                             |
//|   The buffer numbers are the following:                          |
//|      MQL4 0 - MODE_MAIN,  1 - MODE_UPPER, 2 - MODE_LOWER         | ???
//|      MQL5 0 - UPPER_LINE, 1 - LOWER_LINE,        -/-             |
//+------------------------------------------------------------------+
double   iEnvelopes(
                    string                symbol,              // symbol name
                    ENUM_TIMEFRAMES       timeframe,           // timeframe
                    int                   ma_period,           // period for the average line calculation
                    int                   ma_shift,            // horizontal shift of the indicator
                    ENUM_MA_METHOD        ma_method,           // type of smoothing
                    ENUM_APPLIED_PRICE    applied_price,       // type of price
                    double                deviation,           // deviation of boundaries from the midline (in percents)
                    int                   buffer,              // buffer
                    int                   shift                // shift
                    )
  {
//--- it is assumed that the input parameters are written in MQL4 transcription
   switch(buffer)
     {
      case  1:       // MQL4: MODE_UPPER
         buffer=0;   // MQL5: UPPER_LINE
         break;
      case  2:       // MQL4: MODE_LOWER
         buffer=1;   // MQL5: LOWER_LINE
         break;
     }
   double result=NaN;
//---
   int handle=iEnvelopes(symbol,timeframe,ma_period,ma_shift,ma_method,applied_price,deviation);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,buffer,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iMA function in MQL4 notation                                    |
//+------------------------------------------------------------------+
double   iForce(
                string                    symbol,              // symbol name
                ENUM_TIMEFRAMES           timeframe,           // timeframe
                int                       ma_period,           // averaging period
                ENUM_MA_METHOD            ma_method,           // smoothing type
                ENUM_APPLIED_VOLUME       applied_volume,      // volume type for calculation
                int                       shift                // shift
                )
  {
   double result=NaN;
//---
   int handle=iForce(symbol,timeframe,ma_period,ma_method,applied_volume);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iFractals function in MQL4 notation                              |
//|   The buffer numbers are the following:                          |
//|      MQL4 1 - MODE_UPPER, 2 - MODE_LOWER                         |
//|      MQL5 0 - UPPER_LINE, 1 - LOWER_LINE                         |
//+------------------------------------------------------------------+
double   iFractals(
                   string                 symbol,              // symbol name
                   ENUM_TIMEFRAMES        timeframe,           // timeframe
                   int                    buffer,              // buffer
                   int                    shift                // shift
                   )
  {
//--- it is assumed that the input parameters are written in MQL4 transcription
   switch(buffer)
     {
      case  1:       // MQL4: MODE_UPPER
         buffer=0;   // MQL5: UPPER_LINE
         break;
      case  2:       // MQL4: MODE_LOWER
         buffer=1;   // MQL5: LOWER_LINE
         break;
     }
   double result=NaN;
//---
   int handle=iFractals(symbol,timeframe);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,buffer,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iFrAMA function in MQL4 notation                                 |
//+------------------------------------------------------------------+
double   iFrAMA(
                string                    symbol,              // symbol name
                ENUM_TIMEFRAMES           timeframe,           // timeframe
                int                       ma_period,           // averaging period
                int                       ma_shift,            // horizontal shift on the chart
                ENUM_APPLIED_PRICE        applied_price,       // type of price
                int                       shift                // shift
                )
  {
   double result=NaN;
//---
   int handle=iFrAMA(symbol,timeframe,ma_period,ma_shift,applied_price);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iGator function in MQL4 notation                                 |
//|   The buffer numbers are the following:                          |
//|      MQL4 1 - MODE_UPPER, 2 - MODE_LOWER                         |
//|                                                                  |
//|      MQL5 0 - UPPER_HISTOGRAM,                                   |
//|      MQL5 1 - color buffer of the upper histogram,               |
//|      MQL5 2 - LOWER_HISTOGRAM,                                   |
//|      MQL5 3 - color buffer of the lower histogram                |
//+------------------------------------------------------------------+
double   iGator(
                string                    symbol,              // symbol name
                ENUM_TIMEFRAMES           timeframe,           // timeframe
                int                       jaw_period,          // period for the calculation of the jaws
                int                       jaw_shift,           // jaws horizontal shift
                int                       teeth_period,        // period for the calculation of the teeth
                int                       teeth_shift,         // teeth horizontal shift
                int                       lips_period,         // period for the calculation of the lips
                int                       lips_shift,          // lips horizontal shift
                ENUM_MA_METHOD            ma_method,           // type of smoothing
                ENUM_APPLIED_PRICE        applied_price,       // type of price
                int                       buffer,              // buffer
                int                       shift                // shift
                )
  {
//--- it is assumed that the input parameters are written in MQL4 transcription
   switch(buffer)
     {
      case  1:       // MQL4: MODE_UPPER
         buffer=0;   // MQL5: UPPER_HISTOGRAM
         break;
     }
   double result=NaN;
//---
   int handle=iGator(symbol,timeframe,jaw_period,jaw_shift,teeth_period,teeth_shift,lips_period,lips_shift,
                     ma_method,applied_price);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,buffer,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iIchimoku function in MQL4 notation                              |
//|   The buffer numbers are the following:                          |
//|      MQL4 1 - MODE_TENKANSEN                                     |
//|      MQL4 2 - MODE_KIJUNSEN                                      |
//|      MQL4 3 - MODE_SENKOUSPANA                                   |
//|      MQL4 4 - MODE_SENKOUSPANB                                   |
//|      MQL4 5 - MODE_CHIKOUSPAN                                    |
//|                                                                  |
//|      MQL5 0 - TENKANSEN_LINE                                     |
//|      MQL5 1 - KIJUNSEN_LINE                                      |
//|      MQL5 2 - SENKOUSPANA_LINE                                   |
//|      MQL5 3 - SENKOUSPANB_LINE                                   |
//|      MQL5 4 - CHIKOUSPAN_LINE                                    |
//+------------------------------------------------------------------+
double   iIchimoku(
                   string                 symbol,              // symbol name
                   ENUM_TIMEFRAMES        timeframe,           // timeframe
                   int                    tenkan_sen,          // period of Tenkan-sen
                   int                    kijun_sen,           // period of Kijun-sen
                   int                    senkou_span_b,       // period of Senkou Span B
                   int                    buffer,              // buffer
                   int                    shift                // shift
                   )
  {
//--- it is assumed that the input parameters are written in MQL4 transcription
   switch(buffer)
     {
      case  1:       // MQL4: MODE_TENKANSEN
         buffer=0;   // MQL5: TENKANSEN_LINE
         break;
      case  2:       // MQL4: MODE_KIJUNSEN
         buffer=1;   // MQL5: KIJUNSEN_LINE
         break;
      case  3:       // MQL4: MODE_SENKOUSPANA
         buffer=2;   // MQL5: SENKOUSPANA_LINE
         break;
      case  4:       // MQL4: MODE_SENKOUSPANB
         buffer=3;   // MQL5: SENKOUSPANB_LINE
         break;
      case  5:       // MQL4: MODE_CHIKOUSPAN
         buffer=4;   // MQL5: CHIKOUSPAN_LINE
         break;
     }
   double result=NaN;
//---
   int handle=iIchimoku(symbol,timeframe,tenkan_sen,kijun_sen,senkou_span_b);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,buffer,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iBWMFI function in MQL4 notation                                 |
//+------------------------------------------------------------------+
double   iBWMFI(
                string                    symbol,              // symbol name
                ENUM_TIMEFRAMES           timeframe,           // timeframe
                ENUM_APPLIED_VOLUME       applied_volume,      // volume type for calculation
                int                       shift                // shift
                )
  {
   double result=NaN;
//---
   int handle=iBWMFI(symbol,timeframe,applied_volume);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iMomentum function in MQL4 notation                              |
//+------------------------------------------------------------------+
double   iMomentum(
                   string                 symbol,              // symbol name
                   ENUM_TIMEFRAMES        timeframe,           // timeframe
                   int                    mom_period,          // averaging period
                   ENUM_APPLIED_PRICE     applied_price,       // type of price
                   int                    shift                // shift
                   )
  {
   double result=NaN;
//---
   int handle=iMomentum(symbol,timeframe,mom_period,applied_price);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iMFI function in MQL4 notation                                   |
//+------------------------------------------------------------------+
double   iMFI(
              string                      symbol,              // symbol name
              ENUM_TIMEFRAMES             timeframe,           // timeframe
              int                         ma_period,           // averaging period
              ENUM_APPLIED_VOLUME         applied_volume,      // volume type for calculation
              int                         shift                // shift
              )
  {
   double result=NaN;
//---
   int handle=iMFI(symbol,timeframe,ma_period,applied_volume);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iMA function in MQL4 notation                                    |
//+------------------------------------------------------------------+
double   iMA(
             string                       symbol,              // symbol name
             ENUM_TIMEFRAMES              timeframe,           // timeframe
             int                          ma_period,           // averaging period
             int                          ma_shift,            // horizontal shift
             ENUM_MA_METHOD               ma_method,           // smoothing type
             ENUM_APPLIED_PRICE           applied_price,       // type of price
             int                          shift                // shift
             )
  {
   double result=NaN;
//---
   int handle=iMA2(symbol,timeframe,ma_period,ma_shift,ma_method,applied_price);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iOsMA function in MQL4 notation                                  |
//+------------------------------------------------------------------+
double   iOsMA(
               string                     symbol,              // symbol name
               ENUM_TIMEFRAMES            timeframe,           // timeframe
               int                        fast_ema_period,     // period for Fast Moving Average
               int                        slow_ema_period,     // period for Slow Moving Average
               int                        signal_period,       // averaging period for their difference
               ENUM_APPLIED_PRICE         applied_price,       // type of price
               int                        shift                // shift
               )
  {
   double result=NaN;
//---
   int handle=iOsMA(symbol,timeframe,fast_ema_period,slow_ema_period,signal_period,applied_price);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iMACD function in MQL4 notation                                  |
//|   The buffer numbers are the following:                          |
//|      MQL4 0 - MODE_MAIN, 1 - MODE_SIGNAL                         |
//|      MQL5 0 - MAIN_LINE, 1 - SIGNAL_LINE                         |
//+------------------------------------------------------------------+
double   iMACD(
               string                     symbol,              // symbol name
               ENUM_TIMEFRAMES            timeframe,           // timeframe
               int                        fast_ema_period,     // period for Fast average calculation
               int                        slow_ema_period,     // period for Slow average calculation
               int                        signal_period,       // period for their difference averaging
               ENUM_APPLIED_PRICE         applied_price,       // type of price or handle
               int                        buffer,              // buffer
               int                        shift                // shift
               )
  {
   double result=NaN;
//---
   int handle=iMACD2(symbol,timeframe,fast_ema_period,slow_ema_period,signal_period,applied_price);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,buffer,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iOBV function in MQL4 notation                                   |
//+------------------------------------------------------------------+
double   iOBV(
              string                      symbol,              // symbol name
              ENUM_TIMEFRAMES             timeframe,           // timeframe
              ENUM_APPLIED_VOLUME         applied_volume,      // volume type for calculation
              int                         shift                // shift
              )
  {
   double result=NaN;
//---
   int handle=iOBV(symbol,timeframe,applied_volume);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iSAR function in MQL4 notation                                   |
//+------------------------------------------------------------------+
double   iSAR(
              string                      symbol,              // symbol name
              ENUM_TIMEFRAMES             timeframe,           // timeframe
              double                      step,                // price increment step - acceleration factor
              double                      maximum,             // maximum value of step
              int                         shift                // shift
              )
  {
   double result=NaN;
//---
   int handle=iSAR(symbol,timeframe,step,maximum);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iRSI function in MQL4 notation                                   |
//+------------------------------------------------------------------+
double   iRSI(
              string                      symbol,              // symbol name
              ENUM_TIMEFRAMES             timeframe,           // timeframe
              int                         ma_period,           // averaging period
              ENUM_APPLIED_PRICE          applied_price,       // type of price
              int                         shift                // shift
              )
  {
   double result=NaN;
//---
   int handle=iRSI(symbol,timeframe,ma_period,applied_price);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iMACD function in MQL4 notation                                  |
//|   The buffer numbers are the following:                          |
//|      MQL4 0 - MODE_MAIN, 1 - MODE_SIGNAL                         |
//|      MQL5 0 - MAIN_LINE, 1 - SIGNAL_LINE                         |
//+------------------------------------------------------------------+
double   iRVI(
              string                      symbol,              // symbol name
              ENUM_TIMEFRAMES             timeframe,           // timeframe
              int                         ma_period,           // averaging period
              int                         buffer,              // buffer
              int                         shift                // shift
              )
  {
   double result=NaN;
//---
   int handle=iRVI(symbol,timeframe,ma_period);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,buffer,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iStdDev function in MQL4 notation                                |
//+------------------------------------------------------------------+
double   iStdDev(
                 string                   symbol,              // symbol name
                 ENUM_TIMEFRAMES          timeframe,           // timeframe
                 int                      ma_period,           // averaging period
                 int                      ma_shift,            // horizontal shift
                 ENUM_MA_METHOD           ma_method,           // smoothing type
                 ENUM_APPLIED_PRICE       applied_price,       // type of price
                 int                      shift                // shift
                 )
  {
   double result=NaN;
//---
   int handle=iStdDev(symbol,timeframe,ma_period,ma_shift,ma_method,applied_price);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iStochastic function in MQL4 notation                            |
//|   The buffer numbers are the following:                          |
//|      MQL4 0 - MODE_MAIN, 1 - MODE_SIGNAL                         |
//|      MQL5 0 - MAIN_LINE, 1 - SIGNAL_LINE                         |
//+------------------------------------------------------------------+
double   iStochastic(
                     string               symbol,              // symbol name
                     ENUM_TIMEFRAMES      timeframe,           // timeframe
                     int                  Kperiod,             // K-period (number of bars for calculations)
                     int                  Dperiod,             // D-period (period of first smoothing)
                     int                  slowing,             // final smoothing
                     ENUM_MA_METHOD       ma_method,           // type of smoothing
                     ENUM_STO_PRICE       price_field,         // stochastic calculation method
                     int                  buffer,              // buffer
                     int                  shift                // shift
                     )
  {
   double result=NaN;
//---
   int handle=iStochastic(symbol,timeframe,Kperiod,Dperiod,slowing,ma_method,price_field);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iTEMA function in MQL4 notation                                  |
//+------------------------------------------------------------------+
double   iTEMA(
               string                     symbol,              // symbol name
               ENUM_TIMEFRAMES            timeframe,           // timeframe
               int                        ma_period,           // averaging period
               int                        ma_shift,            // horizontal shift of indicator
               ENUM_APPLIED_PRICE         applied_price,       // type of price
               int                        shift                // shift
               )
  {
   double result=NaN;
//---
   int handle=iTEMA(symbol,timeframe,ma_period,ma_shift,applied_price);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iTriX function in MQL4 notation                                  |
//+------------------------------------------------------------------+
double   iTriX(
               string                     symbol,              // symbol name
               ENUM_TIMEFRAMES            timeframe,           // timeframe
               int                        ma_period,           // averaging period
               ENUM_APPLIED_PRICE         applied_price,       // type of price
               int                        shift                // shift
               )
  {
   double result=NaN;
//---
   int handle=iTriX(symbol,timeframe,ma_period,applied_price);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iWPR function in MQL4 notation                                   |
//+------------------------------------------------------------------+
double   iWPR(
              string                      symbol,              // symbol name
              ENUM_TIMEFRAMES             timeframe,           // timeframe
              int                         calc_period,         // averaging period
              int                         shift                // shift
              )
  {
   double result=NaN;
//---
   int handle=iWPR(symbol,timeframe,calc_period);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iVIDyA function in MQL4 notation                                 |
//+------------------------------------------------------------------+
double   iVIDyA(
                string                    symbol,              // symbol name
                ENUM_TIMEFRAMES           timeframe,           // timeframe
                int                       cmo_period,          // period for Chande Momentum
                int                       ema_period,          // EMA smoothing period
                int                       ma_shift,            // horizontal shift on the price chart
                ENUM_APPLIED_PRICE        applied_price,       // type of price
                int                       shift                // shift
                )
  {
   double result=NaN;
//---
   int handle=iVIDyA(symbol,timeframe,cmo_period,ema_period,ma_shift,applied_price);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
//| iVolumes function in MQL4 notation                               |
//+------------------------------------------------------------------+
double   iVolumes(
                  string                  symbol,              // symbol name
                  ENUM_TIMEFRAMES         timeframe,           // timeframe
                  ENUM_APPLIED_VOLUME     applied_volume,      // volume type for calculation
                  int                     shift                // shift
                  )
  {
   double result=NaN;
//---
   int handle=iVolumes(symbol,timeframe,applied_volume);
   if(handle==INVALID_HANDLE)
     {
      Print(__FUNCTION__,": INVALID_HANDLE error=",GetLastError());
      return(result);
     }
   double val[1];
   int copied=CopyBuffer(handle,0,shift,1,val);
   if(copied>0)
      result=val[0];
   else
      Print(__FUNCTION__,": CopyBuffer error=",GetLastError());
   return(result);
  }
//+------------------------------------------------------------------+
